//
// Copyright (c) 2022, MindFusion LLC - Bulgaria.
//

using System;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;


namespace MindFusion.Diagramming.WinForms.Samples.CS.Effects
{
	[DefaultEvent("ColorChanged")]
	public class ColorPicker : Control
	{
		public ColorPicker()
		{
			SetStyle(ControlStyles.AllPaintingInWmPaint, true);
			SetStyle(ControlStyles.OptimizedDoubleBuffer, true);
			SetStyle(ControlStyles.UserPaint, true);
			SetStyle(ControlStyles.ResizeRedraw, true);

			color = Color.White;
			borderColor = SystemColors.ControlDark;
		}

		protected override void OnPaint(PaintEventArgs e)
		{
			base.OnPaint(e);

			Rectangle bounds = ClientRectangle;
			bounds.Width -= 1;
			bounds.Height -= 1;

			using (Brush brush = new SolidBrush(color))
				e.Graphics.FillRectangle(brush, bounds);

			if (!Enabled)
			{
				using (Brush dbrush = new SolidBrush(Color.FromArgb(100, Color.White)))
					e.Graphics.FillRectangle(dbrush, bounds);
			}

			using (Pen pen = new Pen(borderColor))
				e.Graphics.DrawRectangle(pen, bounds);
		}

		protected override void OnMouseDown(MouseEventArgs e)
		{
			base.OnMouseDown(e);

			if (!Enabled)
				return;

			ColorDialog dialog = new ColorDialog();
			dialog.Color = color;
			if (dialog.ShowDialog(this) == DialogResult.OK)
				Color = dialog.Color;
		}


		[DefaultValue(typeof(Color), "White")]
		public Color Color
		{
			get { return color; }
			set
			{
				color = value;
				if (ColorChanged != null)
					ColorChanged(this, EventArgs.Empty);
				Invalidate();
			}
		}

		public Color BorderColor
		{
			get { return borderColor; }
			set
			{
				borderColor = value;
				Invalidate();
			}
		}

		private bool ShouldSerializeBorderColor()
		{
			return borderColor != SystemColors.ControlDark;
		}

		private void ResetBorderColor()
		{
			BorderColor = SystemColors.ControlDark;
		}


		public event EventHandler ColorChanged;


		private Color color;
		private Color borderColor;
	}
}
